package com.rudakova.media_archive.google;

import com.google.api.client.auth.oauth2.StoredCredential;
import com.google.api.client.googleapis.auth.oauth2.GoogleAuthorizationCodeFlow;
import com.rudakova.media_archive.model.Account;
import com.rudakova.media_archive.repository.AccountRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.oauth2.client.OAuth2AuthorizedClientService;
import org.springframework.security.oauth2.client.userinfo.OAuth2UserRequest;
import org.springframework.security.oauth2.core.OAuth2AccessToken;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.Map;

@Component
public class GoogleUserInfoResponseClient {
    private static final Logger LOG = LoggerFactory.getLogger(GoogleUserInfoResponseClient.class);

    @Autowired
    private GoogleAuthorizationCodeFlow flow;

    @Autowired
    private OAuth2AuthorizedClientService clientService;

    @Autowired
    private GoogleService googleService;

    @Autowired
    private AccountRepository accountRepository;

    /**
     * Получение данных о пользвателе и запись их в GoogleAuthorizationCodeFlow.
     * @param userInfoRequest запрос на авторизацию.
     * @return набор данных о пользователе.
     */
    Map<String, Object> getUserInfoResponse(OAuth2UserRequest userInfoRequest) {

        OAuth2AccessToken oauthToken = userInfoRequest.getAccessToken();

        try {
            StoredCredential credential = new StoredCredential();
            credential.setAccessToken(oauthToken.getTokenValue());
            credential.setExpirationTimeMilliseconds(oauthToken.getExpiresAt().toEpochMilli());
            flow.getCredentialDataStore().set("user", credential);

            Map<String, Object> data = googleService.getUserInfo();

            Account account =
                    accountRepository.findByEmailAddress((String) data.get("emailAddress")).orElse(new Account());
            account.setEmailAddress((String) data.get("emailAddress"));
            account.setDisplayName((String) data.get("displayName"));
            account.setToken(oauthToken.getTokenValue());
            accountRepository.save(account);

            flow.getCredentialDataStore().delete("user");
            flow.getCredentialDataStore().set(account.getEmailAddress(), credential);

            return data;
        } catch (IOException e) {
            LOG.error("Error in store credential {}", (Object) e.getStackTrace());
        }

        return null;
    }
}
