package com.rudakova.media_archive.repository;

import com.rudakova.media_archive.model.File;
import org.springframework.data.domain.Pageable;
import org.springframework.data.mongodb.repository.MongoRepository;
import org.springframework.data.mongodb.repository.Query;

import java.util.List;
import java.util.Optional;

public interface FileRepository extends MongoRepository<File, String> {

    @Query("{ 'account.emailAddress' : ?0 }")
    List<File> findByAccount(String emailAddress);

    Optional<File> findByPreview(String preview);

    @Query("{ 'account.emailAddress' : ?0, 'parents' : ?1 }")
    List<File> findByAccountAndParents(String emailAddress, String folderId, Pageable pageable);

    @Query(value = "{ 'account.emailAddress' : ?0, 'parents' : ?1 }", count = true)
    int countByAccountAndParent(String emailAddress, String folderId);

    @Query(value = "{ 'account.emailAddress' : ?0, 'name' : { $regex: ?1, $options: 'i' } }")
    List<File> findByName(String emailAddress, String name);

    @Query(value = "{ 'account.emailAddress' : ?0, 'name' : { $regex: ?1, $options: 'i' } }", count = true)
    int countByName(String emailAddress, String name);
}
