package com.rudakova.media_archive.service;

import com.google.api.client.googleapis.auth.oauth2.GoogleAuthorizationCodeFlow;
import com.rudakova.media_archive.model.Account;
import com.rudakova.media_archive.repository.AccountRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.oauth2.client.OAuth2AuthorizedClientService;
import org.springframework.stereotype.Service;

/**
 * Сервис для получения аворизованного пользователя.
 */
@Service
public class AuthService {

    @Autowired
    private AccountRepository accountRepository;

    @Autowired
    private GoogleAuthorizationCodeFlow flow;

    @Autowired
    private OAuth2AuthorizedClientService clientService;

    /**
     * Получение авторизованного пользователя.
     */
    public Account getLoggedInAccount() {
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        if (authentication == null || authentication.getName().equals("anonymousUser")) {
            return null;
        }

        return accountRepository.findByEmailAddress(authentication.getName()).get();
    }

    public String getEmail() {
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        return authentication != null ? authentication.getName() : null;
    }

    public void loggedOutUser() {
        SecurityContextHolder.getContext().setAuthentication(null);
    }
}
