package com.rudakova.media_archive.servlet;

import com.rudakova.media_archive.model.File;
import com.rudakova.media_archive.repository.FileRepository;
import com.rudakova.media_archive.util.FileUtil;
import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Optional;

/**
 * Сервлет для файлов.
 */
public class FileServlet extends HttpServlet {
    private static final Logger LOG = LoggerFactory.getLogger(FileServlet.class);

    private final String folder;

    private final FileRepository fileRepository;

    public FileServlet(FileRepository fileRepository, String folder) {
        this.fileRepository = fileRepository;
        this.folder = folder;
    }

    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException {
        String name, preview;
        try {
            preview = request.getRequestURL().toString();
            name = request.getPathInfo().substring(1, 65);
        } catch (StringIndexOutOfBoundsException ex) {
            LOG.debug("request request.getPathInfo()=" + request.getPathInfo());
            response.sendError(HttpServletResponse.SC_NOT_FOUND); // 404.
            return;
        }

        Optional<File> files = fileRepository.findByPreview(preview);
        if (files.isEmpty()) {
            LOG.debug("File not found in database");
            response.sendError(HttpServletResponse.SC_NOT_FOUND); // 404.
            return;
        }

        java.io.File ioFile = FileUtil.getFile(name, folder);
        response.setContentLength((int) ioFile.length());
        IOUtils.copy(new FileInputStream(ioFile), response.getOutputStream());
    }
}