package com.rudakova.media_archive.web;

import com.rudakova.media_archive.model.File;
import com.rudakova.media_archive.model.UpdateRecord;
import com.rudakova.media_archive.repository.FileRepository;
import com.rudakova.media_archive.repository.UpdateRecordRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.time.Instant;
import java.time.temporal.ChronoUnit;
import java.util.List;

@RestController
@RequestMapping("/notification")
public class NotificationController {
    private static final Logger LOG = LoggerFactory.getLogger(NotificationController.class);

    @Autowired
    private FileRepository fileRepository;

    @Autowired
    private UpdateRecordRepository updateRecordRepository;

    @PostMapping
    public ResponseEntity<String> notification(HttpServletRequest request) {
        String state = request.getHeader("x-goog-resource-state");
        String changed = request.getHeader("x-goog-changed");
        LOG.info("State value: {}", state);

        if (state != null && (state.equals("trash") ||
                        (state.equals("update") && changed.equals("children")) ||
                        state.equals("untrash"))) {
            String uri = request.getHeader("x-goog-resource-uri");
            String id = uri.substring(uri.lastIndexOf("/") + 1, uri.indexOf("?"));
            String type = state.equals("update") ? changed : state;

            List<UpdateRecord> recordOptional =
                    updateRecordRepository.findByFileIdAndType(id, type);

            if (!recordOptional.isEmpty()) {
                return new ResponseEntity<>(HttpStatus.OK);
            }

            UpdateRecord updateRecord = new UpdateRecord();
            saveRecord(updateRecord, id, type);

            LOG.info("Update record: {}", updateRecord);
        }
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @PostMapping("/video")
    public ResponseEntity<String> video(HttpServletRequest request) {
        String state = request.getHeader("x-goog-resource-state");
        String changed = request.getHeader("x-goog-changed");
        LOG.info("State value: {}", state);

        if (state != null && (state.equals("update") && changed.equals("properties"))) {
            String id = getId(request);

            List<UpdateRecord> updateRecordList = updateRecordRepository.findByFileIdAndType(id, changed);
            UpdateRecord updateRecord = updateRecordList.isEmpty() ? new UpdateRecord() : updateRecordList.get(0);
            saveRecord(updateRecord, id, changed);
            LOG.info("Update record: {}", updateRecord);
        }
        return new ResponseEntity<>(HttpStatus.OK);
    }

    private String getId(HttpServletRequest request) {
        String uri = request.getHeader("x-goog-resource-uri");
        return uri.substring(uri.lastIndexOf("/") + 1, uri.indexOf("?"));
    }

    private void saveRecord(UpdateRecord updateRecord, String id, String type) {
        updateRecord.setFileId(id);
        File file = fileRepository.findById(id).orElse(null);
        updateRecord.setAccount(file == null ? null : file.getAccount());
        updateRecord.setType(type);
        updateRecord.setCreationDate(Instant.now().minus(10, ChronoUnit.MINUTES));
        updateRecordRepository.save(updateRecord);
    }
}
